"""
TODO: This module contains functionlaity to interact with the various
table functions that have been defined in the Snowflake database. They largely
seem to be wrappers around querying the database for a particular time period
so may not be necessary to implement.
"""
